@@ -1,29 +1,21 @@
 /*
  * TransFig: Facility for Translating Fig code
- * Copyright (c) 1985 Supoj Sutantavibul
- * Copyright (c) 1991 Micah Beck
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-1999 by Brian V. Smith
  *
- * THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
- *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons who receive
  * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * that this copyright notice remain intact.
+ *
  */
 
+
 /* 
  *	genibmgl.c :	IBMGL driver for fig2dev
  *			IBM 6180 Color Plotter with
@@ -50,11 +42,8 @@ static set_style();
 #else
 #include <strings.h>
 #endif
-#include <stdio.h>
-#include <math.h>
 #include "fig2dev.h"
 #include "object.h"
-#include "pi.h"
 
 #define		FONTS 			35
 #define		COLORS 			8
@@ -70,11 +59,11 @@ static set_style();
 #define		SPEED_LIMIT		128.0	/* centimeters/second	*/
 
 #ifdef IBMGEC
-static	int	ibmgec		 = TRUE;
+static	int	ibmgec		 = True;
 #else
-static	int	ibmgec		 = FALSE;
+static	int	ibmgec		 = False;
 #endif
-static	int	reflected	 = FALSE;
+static	int	reflected	 = False;
 static	int	fonts		 = FONTS;
 static	int	colors		 = COLORS;
 static	int	patterns	 = PATTERNS;
@@ -204,8 +193,8 @@ char opt, *optarg;
 		break;
 
 	    case 'P':				/* portrait mode	*/
-		landscape	 = FALSE;
-		orientspec	 = TRUE;	/* user-specified	*/
+		landscape	 = False;
+		orientspec	 = True;	/* user-specified	*/
 		break;
 
 	    case 'S':				/* select pen velocity	*/
@@ -213,22 +202,19 @@ char opt, *optarg;
 		break;
 
 	    case 'v':
-		reflected	 = TRUE;	/* mirror image		*/
+		reflected	 = True;	/* mirror image		*/
 		break;
 
 	    default:
 		put_msg(Err_badarg, opt, "ibmgl");
 		exit(1);
-		break;
 	}
 }
 
-static double		ppi;			/*     pixels/inch	*/
 static double		cpi;			/*       cent/inch	*/
 static double		cpp;			/*       cent/pixel	*/
 static double		wcmpp	 = CMPP;	/* centimeter/point	*/
 static double		hcmpp	 = CMPP;	/* centimeter/point	*/
-static int		flipped	 = FALSE;	/* flip Y coordinate	*/
 
 void genibmgl_start(objects)
 F_compound	*objects;
@@ -284,8 +270,6 @@ F_compound	*objects;
 	ppi	 = objects->nwcorner.x;
 	cpi	 = mag*100.0/sqrt((xu-xl)*(xu-xl) + (yu-yl)*(yu-yl));
 	cpp	 = cpi/ppi;
-	if (objects->nwcorner.y == 2)
-	    flipped	 = TRUE;
 
 	/* IBMGL start */
 	fprintf(tfp, "IN;\n");			/* initialize plotter	*/
@@ -302,16 +286,14 @@ F_compound	*objects;
 	    P2x		 = Xur;
 	    if (reflected)			/* upside-down text	*/
 		hcmpp	 = -hcmpp;
-	    if (reflected^flipped) {		/* reflected or flipped */
-		P1y	 = Yur;			/* but not both		*/
-		P2y	 = Yll;
-		}
-	    else {				/* normal		*/
+	    if (reflected) {			/* reflected */
 		P1y	 = Yll;
 		P2y	 = Yur;
-		}
+	    } else {
+		P1y	 = Yur;
+		P2y	 = Yll;
 	    }
-	else {					/* landscape mode	*/
+	} else {					/* landscape mode	*/
 	    Xll	 = xl*UNITS_PER_INCH;
 	    Yll	 = yl*UNITS_PER_INCH;
 	    Yur	 = yu*UNITS_PER_INCH;
@@ -322,20 +304,13 @@ F_compound	*objects;
 		wcmpp	 = -wcmpp;		/* backward text	*/
 		P1x	 = Xur;
 		P2x	 = Xll;
-		}
-	    else {				/* normal		*/
+	    } else {				/* normal		*/
 		P1x	 = Xll;
 		P2x	 = Xur;
-		}
-	    if (flipped) {			/* reflected or not	*/
-		P1y	 = Yur;
-		P2y	 = Yll;
-		}
-	    else {
-		P1y 	 = Yll;
-		P2y 	 = Yur;
-		}
 	    }
+	    P1y	 = Yur;
+	    P2y	 = Yll;
+	}
 
 	Xmin	 = xz;
 	Ymin	 = yz;
@@ -570,17 +545,16 @@ F_arc	*a;
 	    dy2		 = ey - cy;
 	    
 	    theta	 = atan2(dy2, dx2) - atan2(dy1, dx1);
-	    if (a->direction^flipped) {
-		if (theta < 0.0)
-		    theta	+= 2.0*M_PI;
-		}
-	    else {
+	    if (a->direction) {
 		if (theta > 0.0)
 		    theta	-= 2.0*M_PI;
-		}
+	    } else {
+		if (theta < 0.0)
+		    theta	+= 2.0*M_PI;
+	    }
 
-	    if (a->thickness != 0 && a->back_arrow) {
-		arc_tangent(cx, cy, sx, sy, a->direction^flipped, &x, &y);
+	    if (a->type == T_OPEN_ARC && a->thickness != 0 && a->back_arrow) {
+		arc_tangent(cx, cy, sx, sy, !a->direction, &x, &y);
 		draw_arrow_head(x, y, sx, sy,
 		a->back_arrow->ht/ppi, a->back_arrow->wid/ppi);
 		}
@@ -592,8 +566,8 @@ F_arc	*a;
 	    if (a->thickness != 0)
 		fprintf(tfp, "EP;\n");
 
-	    if (a->thickness != 0 && a->for_arrow) {
-		arc_tangent(cx, cy, ex, ey, !a->direction^flipped, &x, &y);
+	    if (a->type == T_OPEN_ARC && a->thickness != 0 && a->for_arrow) {
+		arc_tangent(cx, cy, ex, ey, a->direction, &x, &y);
 		draw_arrow_head(x, y, ex, ey,
 			a->for_arrow->ht/ppi, a->for_arrow->wid/ppi);
 		}
@@ -624,8 +598,8 @@ F_ellipse	*e;
 	    b		 = e->radiuses.y/ppi;
 	    x0		 = e->center.x/ppi;
 	    y0		 = e->center.y/ppi;
-	    angle	 = (flipped ? -e->angle: e->angle);
-	    delta	 = (flipped ? -DELTA: DELTA);
+	    angle	 = -e->angle;
+	    delta	 = -DELTA;
 
 	    x		 = x0 + cos(angle)*a;
 	    y		 = y0 + sin(angle)*a;
@@ -726,18 +700,10 @@ F_line	*l;
 		    x0	 = llx/ppi;
 		    x1	 = urx/ppi;
 		    dx	 = l->radius/ppi;
-		    if (flipped) {
-			y0	 = ury/ppi;
-			y1	 = lly/ppi;
-			dy	 = -dx;
-			angle	 = -M_PI/2.0;
-			}
-		    else {
-			y0	 = lly/ppi;
-			y1	 = ury/ppi;
-			dy	 =  dx;
-			angle	 =  M_PI/2.0;
-			}
+		    y0	 = ury/ppi;
+		    y1	 = lly/ppi;
+		    dy	 = -dx;
+		    angle = -M_PI/2.0;
 
 		    fprintf(tfp, "PA%.4f,%.4f;PM;PD;\n",  x0, y0 + dy);
 		    arc(x0, y0 + dy, x0 + dx, y0 + dy, angle, DELTA);
@@ -758,6 +724,7 @@ F_line	*l;
 		    break;
 
 		case	T_PIC_BOX:
+		    fprintf(stderr,"Warning: Pictures not supported in IBMGL language\n");
 		    break;
 		}
 	    }
@@ -990,10 +957,14 @@ static	double	angle	 = 0.0;		/* label direction  in radians	*/
 	fprintf(tfp, "LB%s\003\n", t->cstring);
 	}
 
-void genibmgl_end()
+int
+genibmgl_end()
 {
 	/* IBMGL ending */
 	fprintf(tfp, "PU;SP;IN;\n");
+
+	/* all ok */
+	return 0;
 }
 
 struct driver dev_ibmgl = {
