@@ -1,33 +1,33 @@
 /*
  * TransFig: Facility for Translating Fig code
- * Copyright (c) 1985 Supoj Sutantavibul
- * Copyright (c) 1991 Micah Beck
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-1999 by Brian V. Smith
  *
- * THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE AUTHORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
- *
- * The X Consortium, and any party obtaining a copy of these files from
- * the X Consortium, directly or indirectly, is granted, free of charge, a
+ * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons who receive
  * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * that this copyright notice remain intact.
+ *
  */
 
+#include <stdlib.h>
+#include <stdio.h>
+#include "patchlevel.h"
+#include <math.h>
+#include "pi.h"
+
+/* location for temporary files */
+#define TMPDIR "/tmp"
+
 typedef char Boolean;
 #define	NO	2
-#define	FALSE	0
-#define	TRUE	1
+#define	False	0
+#define	True	1
 
 #define DEFAULT_FONT_SIZE 11
 
@@ -53,6 +53,10 @@ extern char *strchr();
 #endif
 #endif
 
+#if defined(hpux) || defined(SYSV) || defined(SVR4)
+#define bzero(s,n) memset((s),'\0',(n))
+#endif
+
 extern	double	atof();
 
 #define round(x)	((int) ((x) + ((x >= 0)? 0.5: -0.5)))
@@ -64,6 +68,13 @@ extern	double	atof();
 #define NUMTINTS	20
 #define NUMPATTERNS     22
 
+#ifndef RGB_H
+#define RGB_H
+typedef struct _RGB {
+	unsigned short red, green, blue;
+	} RGB;
+#endif /* RGB_H */
+
 /* 
  * Device driver interface structure
  */
@@ -75,14 +86,13 @@ struct driver {
 	void (*line)();
 	void (*spline)();
 	void (*text)();
-	void (*end)();		/* output file trailer */
+	int (*end)();		/* output file trailer (returns status) */
   	int text_include;	/* include text length in bounding box */
 #define INCLUDE_TEXT 1
 #define EXCLUDE_TEXT 0
 };
 
 extern char Err_badarg[];
-extern char Err_incomp[];
 extern char Err_mem[];
 
 extern char *PSfontnames[];
@@ -95,6 +105,7 @@ extern int	font_size;
 extern double	mag;
 extern FILE	*tfp;
 
+extern int	ppi;		/* Fig file resolution (e.g. 1200) */
 extern int	llx, lly, urx, ury;
 extern Boolean	landscape;
 extern Boolean	center;
@@ -102,12 +113,18 @@ extern Boolean	multi_page;	/* multiple page option for PostScript */
 extern Boolean	orientspec;	/* true if the command-line args specified land or port */
 extern Boolean	centerspec;	/* true if the command-line args specified -c or -e */
 extern Boolean	magspec;	/* true if the command-line args specified -m */
+extern Boolean	transspec;	/* set if the user specs. the GIF transparent color */
 extern Boolean	paperspec;	/* true if the command-line args specified -z */
 extern Boolean	multispec;	/* true if the command-line args specified -M */
 extern Boolean	metric;		/* true if the file contains Metric specifier */
+extern char	gif_transparent[]; /* GIF transp color hex name (e.g. #ff00dd) */
 extern char	papersize[];	/* paper size */
 extern float	THICK_SCALE;	/* convert line thickness from screen res. */
 extern char	lang[];		/* selected output language */
+extern char	*Fig_color_names[]; /* hex names for Fig colors */
+extern RGB	background;	/* background (if specified by -g) */
+extern Boolean	bgspec;		/* flag to say -g was specified */
+extern char	gscom[];	/* to build up a command for ghostscript */
 
 struct paperdef
 {
@@ -116,7 +133,7 @@ struct paperdef
     int height;			/* paper height in points */
 };
 
-#define NUMPAPERSIZES 14
+#define NUMPAPERSIZES 29
 extern struct paperdef paperdef[];
 
 /* user-defined colors */
@@ -130,7 +147,8 @@ extern int		user_col_indx[MAX_USR_COLS];
 extern int		num_usr_cols;
 extern Boolean		pats_used, pattern_used[NUMPATTERNS];
 
-extern void gendev_null();
+extern void	gendev_null();
+extern void	gs_broken_pipe();
 
 /* for GIF files */
 #define	MAXCOLORMAPSIZE 256
@@ -167,6 +185,13 @@ struct Cmap {
 extern int		sys_nerr, errno;
 #endif
 
-#if (! (defined(BSD) && (BSD >= 199306)) && !defined(__NetBSD__))  && !defined(__GNU_LIBRARY__)
-	extern char *sys_errlist[];
+#if ( !(defined(BSD) && (BSD >= 199306)) && !defined(__NetBSD__) && \
+	!defined(__GNU_LIBRARY__) && !defined(__FreeBSD__) && !defined(__GLIBC__))
+	    extern char *sys_errlist[];
 #endif
+
+typedef struct _point
+{
+    int x,y;
+} Point;
+
